#!/bin/bash
#------------------------------------------------------------------------------
set -u

# Usage: 
# bootstrap-linux.sh [<FQDN>] [<SDPBranch>]
# The <SDPBranch> parameter is optional.  SDPBranch determines
# the branch of the SDP to test with, 'main' by default but can
# be 'dev'.

# Bootstraps a new Linux server with requirements Helix Installer testing.

function msg () { echo -e "$*"; }
function errmsg () { msg "\\nBootstrap Error: ${1:-Unknown Error}\\n"; ErrorCount+=1; }
function bail () { msg "\\nError: ${1:-Unknown Error}\\n"; exit "${2:-1}"; }

declare ThisScript="${0##*/}"
declare Version=1.6.3
declare SDPBranch="${2:-main}"
declare HelixInstallerArgs=
declare FQDN="${1:-Unset}"
declare -i ErrorCount=0

msg "Starting $ThisScript v$Version as $USER@${HOSTNAME%%.*} at $(date) as:\\n$ThisScript $*"

# Currently, RHEL/CentOS and Ubuntu flavors are supported.
# See release notes for more specific details of which versions are
# currently supported.
if [[ -r "/etc/redhat-release" ]]; then
   msg "Verified: Running Red Hat or similar Linux OS."
elif [[ -r "/etc/lsb-release" ]]; then
   msg "Verified: Running Ubuntu or similar Linux OS."
else
   msg "Warning: Running on unknown OS. Hoping for the best."
fi

[[ -d /hxdepots/reset ]] && \
   bail "Directory /hxdepots/reset unexpectedly exists at bootstrap!"

if [[ -d /hxdepots ]]; then
   msg "Using existing /hxdepots volume provisioned at bootstrap."
else
   msg "Initializing /hxdepots dir."
   if ! sudo mkdir /hxdepots; then
      errmsg "MKDIR of /hxepots failed."
   fi
fi

[[ -d /depotdata/reset ]] && \
      bail "Directory /depotdata/reset unexpectedly exists at bootstrap!"

if [[ -d /depotdata ]]; then
   msg "Using existing /depotdata volume provisioned at bootstrap."
else
   msg "Initializing /depotdata dir."
   if ! sudo mkdir /depotdata; then
      errmsg "MKDIR of /depotdata failed."
   fi
fi

# Some tests use the default /hxdepots mount point directory, while
# others test the ability to configure an alternative, /depotdata.
# In this bootstrap script, we ensure both are available.

if [[ -d /tmp/test-reset ]]; then
   msg "Testing files from local workspace."

   if sudo cp -pr /tmp/test-reset /hxdepots/reset; then
      msg "Copied /tmp/test-reset to /hxdepots/reset"
   else
      errmsg "Failed to copy /tmp/test-reset to /hxdepots/reset"
   fi

   if sudo cp -pr /tmp/test-reset /depotdata/reset; then
      msg "Copied /tmp/test-reset to /depotdata/reset"
   else
      errmsg "Failed to copy /tmp/test-reset to /depotdata/reset"
   fi

   cd /hxdepots/reset || errmsg "Failed to do 'cd /hxdepots/reset'. Aborting."
   cd /depotdata/reset || errmsg "Failed to do 'cd /depotdata/reset'. Aborting."
else
   bail "Expected test support dir /tmp/test-reset does not exist. Check the Vagrantfile for the setting 'machine.vm.synced_folder'."
fi

if [[ "$FQDN" != "Unset" ]]; then
   if [[ -n "$(command -v hostnamectl)" ]]; then
      msg "Setting hostname using: hostnamectl set-hostname $FQDN"
      hostnamectl set-hostname "$FQDN"
   else
      msg "Setting hostname in /etc/sysconfig/network and calling: hostname $FQDN"
      echo -e "NETWORKING=yes\\nHOSTNAME=$FQDN\\n" > /etc/sysconfig/network
      hostname "$FQDN"
   fi
fi

### WORKAROUND for incompatibility with SELinux in enforcing mode on
### RHEL/CentOS 8.
### See: https://swarm.workshop.perforce.com/jobs/SDP-562
### See: https://swarm.workshop.perforce.com/jobs/HI-54
if [[ -r "/etc/redhat-release" ]]; then
   if grep -qi "Linux release 8" /etc/redhat-release; then
      msg "Disabling SELinux on host ${HOSTNAME%%.*}."

      if setenforce 0; then
         msg "SELinux disabled for this session."
      else
         errmsg "Failed to disable SELinux for this session."
      fi

      if sed 's:^SELINUX=enforcing:SELINUX=permissive:g' /etc/selinux/config > /etc/selinux/config.new; then
         if mv -f /etc/selinux/config.new /etc/selinux/config; then
            msg "SELinux permanently disabled."
         else
            errmsg "Failed to disable SELinux permanently."
         fi
      else
         errmsg "Failed to run 'sed' in /etc/selinux/config file."
      fi
   fi
fi

HelixInstallerVersion=$(grep '^declare Version=' reset_sdp.sh)
HelixInstallerVersion=${HelixInstallerVersion#declare }
msg "Helix Installer version: $HelixInstallerVersion"

HelixInstallerCmd="./reset_sdp.sh"
HelixInstallerArgs="-fast -b $SDPBranch -M"
#HelixInstallerArgs="-b $SDPBranch -M"
msg "Calling Helix Installer: $HelixInstallerCmd $HelixInstallerArgs"

#shellcheck disable=SC2086,SC2024
if ! sudo $HelixInstallerCmd $HelixInstallerArgs > log.reset_sdp.txt 2>&1; then
   errmsg "Sudo command failed: $HelixInstallerCmd $HelixInstallerArgs"
fi

exit "$ErrorCount"
